@@ -7,15 +7,11 @@
  *
  */
 
-if (!Object.hasOwn) {
-    Object.hasOwn = Object.call.bind(Object.hasOwnProperty);
-}
-
 const Utilities = {
-  VERSION: "9.0",
-  REVISION: 1285,
-  LOCAL_MODIFICATIONS: 3,
-  BRANCH: "master",
+  VERSION : "17.1",
+  REVISION : 0,
+  LOCAL_MODIFICATIONS: 0,
+  BRANCH: "v17.x",
 
   exc: console_error_safe,
   error: console_error_safe,
@@ -31,7 +27,7 @@ const Utilities = {
   cdebug: console_debug_safe,
 
   stristrue(v, default_value) {
-    if (v === null) {
+    if (!v) {
       return default_value;
     }
     return ["true", "on", "1", "yes", "enabled"].includes(
@@ -39,6 +35,11 @@ const Utilities = {
     );
   },
 
+  removeChars(validChars, inputString) {
+    var regex = new RegExp('[^' + validChars + ']', 'g');
+    return inputString.replace(regex, '');
+  },
+
   getHexUUID() {
     const s = [];
     const hexDigits = "0123456789abcdef";
@@ -59,52 +60,163 @@ const Utilities = {
     return c.charCodeAt(0);
   },
 
-  getSecureRandomString(length_) {
-    const crypto = window.crypto || window.mscrypto;
-    if (!crypto) {
-      let s = "";
-      while (s.length < length_) {
-        s += Utilities.getHexUUID();
-      }
-      return s.slice(0, Math.max(0, length_));
-    }
-    const u = new Uint8Array(length_);
+  getSecureRandomBytes(len) {
+    const u = new Uint8Array(len);
     crypto.getRandomValues(u);
+    return u;
+  },
+
+  getSecureRandomString(len) {
+    const u = Utilities.getSecureRandomBytes(len);
     return String.fromCharCode.apply(null, u);
   },
 
   xorString(string1, string2) {
-    let result = "";
     if (string1.length !== string2.length) {
       throw "strings must be equal length";
     }
-    for (const index in string1) {
+    let result = "";
+    for (let index = 0; index < string1.length; index++) {
       const character1 = string1[index];
       const character2 = string2[index];
-      result += String.fromCharCode(
-        character1.charCodeAt(0) ^ character2.charCodeAt(0)
-      );
+      result += String.fromCharCode(character1.charCodeAt(0) ^ character2.charCodeAt(0));
     }
     return result;
   },
 
+  u8(value) {
+    const type = typeof value;
+    if (type === 'object' && value.constructor === Uint8Array) {
+      return value;
+    }
+    if (type == "string") {
+      return Uint8Array.from(value.split("").map(x => x.charCodeAt()));
+    }
+    return new Uint8Array(value);
+  },
+
+  xor(str1, str2) {
+    const trimmed_str2 = str2.slice(0, str1.length);
+    return Utilities.xorString(str1, trimmed_str2);
+  },
+
+  gendigest(digest, password, salt) {
+    // Utilities.clog("gendigest(", digest, ", ", password, ", ", salt, ")");
+    // Utilities.clog("gendigest(", digest, ", ", Utilities.convertToHex(password), ", ", Utilities.convertToHex(salt), ")");
+    if (digest == "xor") {
+      const trimmed_salt = salt.slice(0, password.length);
+      // Utilities.debug("xoring with trimmed salt:", Utilities.convertToHex(trimmed_salt));
+      const promise = new Promise(function(resolve, reject) {
+        const xored = Utilities.xorString(trimmed_salt, password);
+        resolve(xored);
+      });
+      return promise;
+    }
+    if (!digest.startsWith("hmac")) {
+      return new Promise(function(resolve, reject) {
+        reject(new Error("unsupported digest '" + digest + "'"));
+      });
+    }
+    let hash = "SHA-1";
+    if (digest.indexOf("+") > 0) {
+      // "hmac+sha512" -> "sha512"
+      hash = digest.split("+")[1];
+    }
+    hash = hash.toUpperCase();
+    if (hash.startsWith("SHA") && !hash.startsWith("SHA-")) {
+      hash = "SHA-" + hash.substring(3);
+    }
+
+    if (typeof crypto.subtle === "undefined") {
+      // use hmac.js
+      return new Promise(function(resolve, reject) {
+        if (hash != "SHA-256") {
+          reject(new Error("crypto.subtle API is not available in this context"));
+        }
+        else {
+          value = hmac(Utilities.u8(password), Utilities.u8(salt));
+          resolve(value);
+        }
+      });
+    }
+
+    const promise = new Promise(function(resolve, reject) {
+      Utilities.clog("crypto.subtle=", crypto.subtle);
+      Utilities.clog("crypto.subtle.importKey=", crypto.subtle.importKey);
+      crypto.subtle.importKey("raw", Utilities.u8(password), {
+          name: "HMAC",
+          hash: hash
+        }, false, ["sign", "verify"])
+        .then(
+          (key) => {
+            Utilities.cdebug("imported hmac key: ", key);
+            const u8salt = Utilities.u8(salt);
+            crypto.subtle.sign("HMAC", key, u8salt).then(
+              (result) => {
+                const u8digest = new Uint8Array(result);
+                Utilities.clog("hmac result=", u8digest);
+                resolve(Utilities.ArrayBufferToString(u8digest));
+              },
+              (error) => {
+                Utilities.clog("hmac signing failed:", error);
+                reject(error);
+              }
+            );
+          },
+          (error) => {
+            Utilities.clog("hmac key import failed:", error);
+            reject(error);
+          }
+        );
+    });
+    return promise;
+  },
+
   trimString(string_, trimLength) {
     if (!string_) {
       return "";
     }
-    return string_.length > trimLength
-      ? `${string_.slice(0, Math.max(0, trimLength - 3))}...`
-      : string_;
+    return string_.length > trimLength ?
+      `${string_.slice(0, Math.max(0, trimLength - 3))}...` :
+      string_;
   },
 
-  convertToHex(string_) {
+  convertToHex(value) {
+    if (value instanceof Uint8Array) {
+      return Utilities.arrayhex(value);
+    }
     let hex = "";
-    for (let index = 0; index < string_.length; index++) {
-      hex += `${string_.charCodeAt(index).toString(16).padStart(2, "0")}`;
+    for (let index = 0; index < value.length; index++) {
+      hex += `${value.charCodeAt(index).toString(16).padStart(2, "0")}`;
     }
     return hex;
   },
 
+  hexToString(hexval) {
+    var hex = hexval.toString();
+    var str = '';
+    for (var i = 0; i < hex.length; i += 2) {
+      str += String.fromCharCode(parseInt(hex.substr(i, 2), 16));
+    }
+    return str;
+  },
+
+  arrayhex(arr) {
+    return Array.from(arr).map((b) => b.toString(16).padStart(2, "0")).join("");
+  },
+
+  arrayToHex(buffer) {
+    return Array.prototype.map.call(buffer, x => ('00' + x.toString(16)).slice(-2)).join('');
+  },
+
+  hexarray(hex) {
+    var bytes = new Uint8Array(Math.ceil(hex.length / 2));
+    for (var i = 0; i < bytes.length; i++) {
+      bytes[i] = parseInt(hex.substr(i * 2, 2), 16);
+    }
+    return bytes;
+  },
+
   getPlatformProcessor() {
     //mozilla property:
     if (navigator.oscpu) {
@@ -343,7 +455,7 @@ const Utilities = {
 
   //https://github.com/facebook/fixed-data-table/blob/master/src/vendor_upstream/dom/normalizeWheel.js
   //BSD license
-  normalizeWheel(/*object*/ event) /*object*/ {
+  normalizeWheel( /*object*/ event) /*object*/ {
     // Reasonable defaults
     const PIXEL_STEP = 10;
     const LINE_HEIGHT = 40;
@@ -451,22 +563,20 @@ const Utilities = {
       return new Uint8Array(0);
     }
     const type = typeof v;
-      if (type === 'object' && v.constructor===Uint8Array) {
-        return v;
-      }
-     return StringToUint8(v.toString());
+    if (type === 'object' && v.constructor === Uint8Array) {
+      return v;
+    }
+    return Utilities.StringToUint8(v.toString());
   },
 
-  ArrayBufferToBase64(uintArray) {
+  ArrayBufferToString(uintArray) {
     // apply in chunks of 10400 to avoid call stack overflow
     // https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Function/apply
     let s = "";
     const skip = 10_400;
     if (uintArray.subarray) {
       for (
-        let index = 0, length_ = uintArray.length;
-        index < length_;
-        index += skip
+        let index = 0, length_ = uintArray.length; index < length_; index += skip
       ) {
         s += String.fromCharCode.apply(
           null,
@@ -475,9 +585,7 @@ const Utilities = {
       }
     } else {
       for (
-        let index = 0, length_ = uintArray.length;
-        index < length_;
-        index += skip
+        let index = 0, length_ = uintArray.length; index < length_; index += skip
       ) {
         s += String.fromCharCode.apply(
           null,
@@ -485,7 +593,11 @@ const Utilities = {
         );
       }
     }
-    return window.btoa(s);
+    return s;
+  },
+
+  ArrayBufferToBase64(uintArray) {
+    return window.btoa(Utilities.ArrayBufferToString(uintArray));
   },
 
   ToBase64(v) {
@@ -540,6 +652,10 @@ const Utilities = {
     return value;
   },
 
+  isSafeHost(host) {
+    return host && ["localhost", "127.0.0.1", "::1"].indexOf(host) >= 0;
+  },
+
   /**
    * XmlHttpRequest's getAllResponseHeaders() method returns a string of response
    * headers according to the format described here:
@@ -580,7 +696,7 @@ const Utilities = {
   getparam(property) {
     let getParameter = window.location.getParameter;
     if (!getParameter) {
-      getParameter = function (key) {
+      getParameter = function(key) {
         if (!window.location.queryStringParams)
           window.location.queryStringParams = Utilities.parseParams(
             window.location.search.slice(1)
@@ -588,20 +704,21 @@ const Utilities = {
         return window.location.queryStringParams[key];
       };
     }
+
     let value = getParameter(property);
-    try {
-      if (value === undefined && typeof sessionStorage !== "undefined") {
-        value = sessionStorage.getItem(property);
-      }
-    } catch {
-      value = null;
+
+    if (value === undefined) {
+      try {
+        value = Utilities.getSessionStorageValue(property);
+      } catch {}
     }
+
     return value;
   },
 
   getboolparam(property, default_value) {
     const v = Utilities.getparam(property);
-    if (v === null) {
+    if (v === undefined) {
       return default_value;
     }
     return ["true", "on", "1", "yes", "enabled"].includes(
@@ -623,6 +740,34 @@ const Utilities = {
     }
   },
 
+  getSessionStorageValue(property) {
+    const params = JSON.parse(sessionStorage.getItem(Utilities.getSessionStoragePrefix()))
+    if (property in params) {
+      return String(params[property]);
+    }
+    return undefined;
+  },
+
+  setSessionStorageValue(property, value) {
+    const prefix = Utilities.getSessionStoragePrefix();
+    let params = JSON.parse(sessionStorage.getItem(prefix)) || {}
+    if (value === null || value === "undefined") {
+      delete params[property];
+    } else {
+      params[property] = String(value);
+    }
+    sessionStorage.setItem(prefix, JSON.stringify(params));
+  },
+
+  clearSessionStorage() {
+    sessionStorage.removeItem(Utilities.getSessionStoragePrefix());
+  },
+
+  getSessionStoragePrefix() {
+    const urlPath = new URL(window.location.href).pathname
+    return urlPath.substring(0, urlPath.lastIndexOf("/"));
+  },
+
   getConnectionInfo() {
     if (!Object.hasOwn(navigator, "connection")) {
       return {};
@@ -675,7 +820,7 @@ const Utilities = {
       xhr.setRequestHeader("Authorization", `Basic ${credentials}`);
     }
     xhr.responseType = "json";
-    xhr.addEventListener("load", function () {
+    xhr.addEventListener("load", function() {
       Utilities.log("loaded", url, "status", xhr.status);
       const status = xhr.status;
       if (status === 200) {
@@ -687,13 +832,13 @@ const Utilities = {
         }
       }
     });
-    xhr.addEventListener("error", function (e) {
+    xhr.addEventListener("error", function(e) {
       Utilities.log(uri, "error:", e);
       if (error_function) {
         error_function(e);
       }
     });
-    xhr.addEventListener("abort", function (e) {
+    xhr.addEventListener("abort", function(e) {
       Utilities.log(uri, "abort:", e);
       if (error_function) {
         error_function(e);
