@@ -98,22 +98,10 @@ class XpraWindow {
     this.icon = null;
 
     // get offsets
-    this.leftoffset = Number.parseInt(
-      jQuery(this.div).css("border-left-width"),
-      10
-    );
-    this.rightoffset = Number.parseInt(
-      jQuery(this.div).css("border-right-width"),
-      10
-    );
-    this.topoffset = Number.parseInt(
-      jQuery(this.div).css("border-top-width"),
-      10
-    );
-    this.bottomoffset = Number.parseInt(
-      jQuery(this.div).css("border-bottom-width"),
-      10
-    );
+    this.leftoffset = Number.parseInt(jQuery(this.div).css("border-left-width"), 10);
+    this.rightoffset = Number.parseInt(jQuery(this.div).css("border-right-width"), 10);
+    this.topoffset = Number.parseInt(jQuery(this.div).css("border-top-width"), 10);
+    this.bottomoffset = Number.parseInt(jQuery(this.div).css("border-bottom-width"), 10);
 
     // update metadata that is safe before window is drawn
     this.update_metadata(metadata, true);
@@ -124,6 +112,8 @@ class XpraWindow {
       jQuery(this.div).addClass(`window-${this.windowtype}`);
     }
 
+    const fullscreen = (metadata["fullscreen"]) ?? false;
+
     if (this.client.server_is_desktop || this.client.server_is_shadow) {
       jQuery(this.div).addClass("desktop");
       this.resizable = false;
@@ -131,12 +121,12 @@ class XpraWindow {
       jQuery(this.div).addClass("tray");
     } else if (this.override_redirect) {
       jQuery(this.div).addClass("override-redirect");
-    } else if (
-      this.windowtype == "" ||
-      this.windowtype == "NORMAL" ||
-      this.windowtype == "DIALOG" ||
-      this.windowtype == "UTILITY"
-    ) {
+    } else if (!fullscreen && (
+        this.windowtype == "" ||
+        this.windowtype == "NORMAL" ||
+        this.windowtype == "DIALOG" ||
+        this.windowtype == "UTILITY"
+      )) {
       this.resizable = true;
     }
 
@@ -203,9 +193,13 @@ class XpraWindow {
     jQuery(this.div).prepend(head);
     // make draggable
     if (this.scale !== 1) {
-      jQuery(this.div).draggable({ transform: true });
+      jQuery(this.div).draggable({
+        transform: true
+      });
     }
-    jQuery(this.div).draggable({ cancel: "canvas" });
+    jQuery(this.div).draggable({
+      cancel: "canvas"
+    });
     jQuery(`#head${wid}`).click((event_) => {
       if (!this.minimized) {
         this.focus();
@@ -224,7 +218,9 @@ class XpraWindow {
     // This disables helper highlight, so we
     // move the resizable borders in transform plugin
     if (this.scale !== 1) {
-      jQuery(this.div).resizable({ transform: true });
+      jQuery(this.div).resizable({
+        transform: true
+      });
     }
     // attach resize handles
     jQuery(this.div).resizable({
@@ -261,8 +257,7 @@ class XpraWindow {
       jQuery(this.d_minimizebtn).hide();
     }
     // adjust top offset
-    this.topoffset =
-      this.topoffset + Number.parseInt(jQuery(this.d_header).css("height"), 10);
+    this.topoffset = this.topoffset + Number.parseInt(jQuery(this.d_header).css("height"), 10);
     // stop propagation if we're over the window:
     jQuery(this.div).mousedown((e) => e.stopPropagation());
     //bug 2418: if we stop 'mouseup' propagation,
@@ -310,8 +305,7 @@ class XpraWindow {
 
   transfer_canvas(canvas) {
     const offscreen_handle = canvas.transferControlToOffscreen();
-    this.client.decode_worker.postMessage(
-      {
+    this.client.decode_worker.postMessage({
         cmd: "canvas",
         wid: this.wid,
         canvas: offscreen_handle,
@@ -362,8 +356,8 @@ class XpraWindow {
         this.pointer_last_y = event_.offsetY;
       }
     });
-    canvas.addEventListener("pointermove", (event_) => {
-      this.debug("mouse", "pointermove:", event_);
+    canvas.addEventListener("mousemove", (event_) => {
+      this.debug("mouse", "mousemove:", event_);
       if (this.pointer_down == event_.pointerId) {
         const dx = event_.offsetX - this.pointer_last_x;
         const dy = event_.offsetY - this.pointer_last_y;
@@ -388,6 +382,7 @@ class XpraWindow {
     });
     //wheel events on a window:
     const me = this;
+
     function on_mousescroll(e) {
       me.on_mousescroll(e);
       e.stopPropagation();
@@ -395,10 +390,6 @@ class XpraWindow {
     }
     if (Utilities.isEventSupported("wheel")) {
       canvas.addEventListener("wheel", on_mousescroll, false);
-    } else if (Utilities.isEventSupported("mousewheel")) {
-      canvas.addEventListener("mousewheel", on_mousescroll, false);
-    } else if (Utilities.isEventSupported("DOMMouseScroll")) {
-      canvas.addEventListener("DOMMouseScroll", on_mousescroll, false); // for Firefox
     }
   }
 
@@ -438,17 +429,7 @@ class XpraWindow {
     } else if (oldy >= wh - min_visible) {
       this.y = Math.min(oldy, wh - min_visible);
     }
-    this.debug(
-      "geometry",
-      "ensure_visible() oldx=",
-      oldx,
-      "oldy=",
-      oldy,
-      "x=",
-      this.x,
-      "y=",
-      this.y
-    );
+    this.debug("geometry", "ensure_visible() oldx=", oldx, "oldy=", oldy, "x=", this.x, "y=", this.y);
     if (oldx != this.x || oldy != this.y) {
       this.updateCSSGeometry();
       return false;
@@ -485,7 +466,13 @@ class XpraWindow {
     // set size of canvas
     this.updateCanvasGeometry();
     if (this.client.server_is_desktop || this.client.server_is_shadow) {
-      jQuery(this.div).position({ of: jQuery("#screen") });
+      if (this.client.server_resize_exact) {
+        jQuery(this.div).css("top", 0);
+      }
+
+      jQuery(this.div).position({
+        of: jQuery("#screen")
+      });
       return;
     }
     // work out outer size
@@ -499,17 +486,7 @@ class XpraWindow {
     this.outerY = this.y - this.topoffset;
     jQuery(this.div).css("left", this.outerX);
     jQuery(this.div).css("top", this.outerY);
-    this.debug(
-      "geometry",
-      "updateCSSGeometry() left=",
-      this.outerX,
-      ", top=",
-      this.outerY,
-      ", width=",
-      this.outerW,
-      ", height=",
-      this.outerH
-    );
+    this.debug("geometry", "updateCSSGeometry() left=", this.outerX, ", top=", this.outerY, ", width=", this.outerW, ", height=", this.outerH);
   }
 
   focus() {
@@ -522,18 +499,18 @@ class XpraWindow {
       jQuery(this.div).addClass("windowinfocus");
 
       // Update window title
-      jQuery("title").text(
-        `${location.pathname.replaceAll("/", "")}: ${this.title}`
-      );
+      if (this.client.session_name) {
+        jQuery("title").text(client.session_name);
+      }
+      else {
+        jQuery("title").text(
+          `${location.pathname.replaceAll("/", "")}: ${this.title}`
+        );
+      }
 
       // Update the icon
       if (this.icon !== null) {
-        const source = this.update_icon(
-          this.icon.width,
-          this.icon.height,
-          this.icon.encoding,
-          this.icon.img_data
-        );
+        const source = this.update_icon(this.icon.width, this.icon.height, this.icon.encoding, this.icon.img_data);
         jQuery("#favicon").attr("href", source);
       } else {
         jQuery("#favicon").attr("href", "favicon.png");
@@ -582,11 +559,7 @@ class XpraWindow {
     let z = 5000 + this.stacking_layer;
     if (this.tray) {
       z = 0;
-    } else if (
-      this.override_redirect ||
-      this.client.server_is_desktop ||
-      this.client.server_is_shadow
-    ) {
+    } else if (this.override_redirect || this.client.server_is_desktop || this.client.server_is_shadow) {
       z = 30_000;
     } else if (
       this.windowtype == "DROPDOWN" ||
@@ -690,11 +663,7 @@ class XpraWindow {
         //remove any existing "wmclass-" classes not in the new wm_class list:
         for (const class_ of classes) {
           const tclass = `${class_}`;
-          if (
-            tclass.indexOf("wmclass-") === 0 &&
-            wm_class &&
-            !wm_class.includes(tclass)
-          ) {
+          if (tclass.indexOf("wmclass-") === 0 && wm_class && !wm_class.includes(tclass)) {
             jQuery(this.div).removeClass(tclass);
           }
         }
@@ -790,11 +759,7 @@ class XpraWindow {
       w: this.w,
       h: this.h,
     };
-    this.debug(
-      "geometry",
-      "save_geometry() saved-geometry=",
-      this.saved_geometry
-    );
+    this.debug("geometry", "save_geometry() saved-geometry=", this.saved_geometry);
   }
   /**
    * Restores the saved geometry (if it exists).
@@ -807,11 +772,7 @@ class XpraWindow {
     this.y = this.saved_geometry["y"];
     this.w = this.saved_geometry["w"];
     this.h = this.saved_geometry["h"];
-    this.debug(
-      "geometry",
-      "restore_geometry() saved-geometry=",
-      this.saved_geometry
-    );
+    this.debug("geometry", "restore_geometry() saved-geometry=", this.saved_geometry);
     // delete saved geometry
     this.saved_geometry = null;
     // then call local resized callback
@@ -868,21 +829,13 @@ class XpraWindow {
     const geom = this.get_internal_geometry();
     this.set_minimized(!this.minimized);
     if (this.minimized) {
-      this.client.send(["unmap-window", this.wid, true]);
+      this.client.send([PACKET_TYPES.unmap_window, this.wid, true]);
       this.stacking_layer = 0;
       if (this.client.focus == this.wid) {
         this.client.auto_focus();
       }
     } else {
-      this.client.send([
-        "map-window",
-        this.wid,
-        geom.x,
-        geom.y,
-        geom.w,
-        geom.h,
-        this.client_properties,
-      ]);
+      this.client.send([PACKET_TYPES.map_window, this.wid, geom.x, geom.y, geom.w, geom.h, this.client_properties]);
       //force focus switch:
       this.client.focus = -1;
       this.client.set_focus(this);
@@ -916,25 +869,14 @@ class XpraWindow {
   }
 
   _set_decorated(decorated) {
-    this.topoffset = Number.parseInt(
-      jQuery(this.div).css("border-top-width"),
-      10
-    );
+    this.topoffset = Number.parseInt(jQuery(this.div).css("border-top-width"), 10);
     if (decorated) {
       jQuery(`#head${this.wid}`).show();
       jQuery(this.div).removeClass("undecorated");
       jQuery(this.div).addClass("window");
       if (this.d_header) {
-        this.topoffset =
-          this.topoffset +
-          Number.parseInt(jQuery(this.d_header).css("height"), 10);
-        this.debug(
-          "geometry",
-          "_set_decorated(",
-          decorated,
-          ") new topoffset=",
-          self.topoffset
-        );
+        this.topoffset = this.topoffset + Number.parseInt(jQuery(this.d_header).css("height"), 10);
+        this.debug("geometry", "_set_decorated(", decorated, ") new topoffset=", self.topoffset);
       }
     } else {
       jQuery(`#head${this.wid}`).hide();
@@ -968,8 +910,7 @@ class XpraWindow {
     this.x = this.leftoffset;
     this.y = this.topoffset;
     this.w = screen_size[0] - this.leftoffset - this.rightoffset;
-    this.h =
-      screen_size[1] - this.topoffset - this.bottomoffset - TASKBAR_HEIGHT;
+    this.h = screen_size[1] - this.topoffset - this.bottomoffset - TASKBAR_HEIGHT;
     this.debug("geometry", "fill_screen() ", this.x, this.y, this.w, this.h);
   }
 
@@ -1020,13 +961,8 @@ class XpraWindow {
    * if it is fullscreen or maximized.
    */
   screen_resized() {
-    this.debug(
-      "geometry",
-      "screen_resized() server_is_desktop=",
-      this.client.server_is_desktop,
-      ", server_is_shadow=",
-      this.client.server_is_shadow
-    );
+    this.debug("geometry", "screen_resized() server_is_desktop=", this.client.server_is_desktop,
+      ", server_is_shadow=", this.client.server_is_shadow);
     if (this.client.server_is_desktop) {
       this.match_screen_size();
     }
@@ -1051,16 +987,7 @@ class XpraWindow {
   recenter(force_update_geometry) {
     let x = this.x;
     let y = this.y;
-    this.debug(
-      "geometry",
-      "recenter() x=",
-      x,
-      ", y=",
-      y,
-      ", desktop size: ",
-      this.client.desktop_width,
-      this.client.desktop_height
-    );
+    this.debug("geometry", "recenter() x=", x, ", y=", y, ", desktop size: ", this.client.desktop_width, this.client.desktop_height);
     x = Math.round((this.client.desktop_width - this.w) / 2);
     y = Math.round((this.client.desktop_height - this.h) / 2);
     if (this.x != x || this.y != y || force_update_geometry) {
@@ -1158,22 +1085,9 @@ class XpraWindow {
     this.move_resize(this.x, this.y, w, h);
   }
 
-  initiate_moveresize(
-    mousedown_event,
-    x_root,
-    y_root,
-    direction,
-    button,
-    source_indication
-  ) {
+  initiate_moveresize(mousedown_event, x_root, y_root, direction, button, source_indication) {
     const dir_str = MOVERESIZE_DIRECTION_STRING[direction];
-    this.log("initiate_moveresize", dir_str, [
-      x_root,
-      y_root,
-      direction,
-      button,
-      source_indication,
-    ]);
+    this.log("initiate_moveresize", dir_str, [x_root, y_root, direction, button, source_indication]);
     if (direction == MOVERESIZE_MOVE && mousedown_event) {
       const e = mousedown_event;
       e.type = "mousedown.draggable";
@@ -1218,17 +1132,6 @@ class XpraWindow {
     };
   }
 
-  /**
-   * Handle mouse click from this window's canvas,
-   * then we fire "mouse_click_cb" (if it is set).
-   */
-  handle_mouse_click(button, pressed, mx, my, modifiers, buttons) {
-    this.debug("mouse", "got mouse click at ", mx, my);
-    // mouse click event is from canvas just for this window so no need to check
-    // internal geometry anymore
-    this.mouse_click_cb(this, button, pressed, mx, my, modifiers, buttons);
-  }
-
   update_icon(width, height, encoding, img_data) {
     // Cache the icon.
     this.icon = {
@@ -1269,6 +1172,7 @@ class XpraWindow {
     const window_element = jQuery(`#${this.wid}`);
     const cursor_url = this.construct_base64_image_url(encoding, img_data);
     const me = this;
+
     function set_cursor_url(url, x, y, w, h) {
       const url_string = `url('${url}')`;
       window_element.css("cursor", `${url_string}, default`);
@@ -1320,12 +1224,7 @@ class XpraWindow {
   draw() {
     //pass the 'buffer' canvas directly to visible canvas context
     if (this.has_alpha || this.tray) {
-      this.canvas_ctx.clearRect(
-        0,
-        0,
-        this.draw_canvas.width,
-        this.draw_canvas.height
-      );
+      this.canvas_ctx.clearRect(0, 0, this.draw_canvas.width, this.draw_canvas.height);
     }
     this.canvas_ctx.drawImage(this.draw_canvas, 0, 0);
   }
@@ -1354,13 +1253,8 @@ class XpraWindow {
    * if we're not already in the process of painting something.
    */
   may_paint_now() {
-    this.debug(
-      "draw",
-      "may_paint_now() paint pending=",
-      this.paint_pending,
-      ", paint queue length=",
-      this.paint_queue.length
-    );
+    this.debug("draw", "may_paint_now() paint pending=", this.paint_pending,
+      ", paint queue length=", this.paint_queue.length);
     let now = performance.now();
     while (
       (this.paint_pending == 0 || now - this.paint_pending >= 2000) &&
@@ -1399,40 +1293,17 @@ class XpraWindow {
     const bitmap = coding.startsWith("bitmap:");
     if (bitmap) {
       coding = coding.split(":")[1];
-      this.debug(
-        "draw",
-        coding,
-        img_data,
-        " at ",
-        `${x},${y}`,
-        ") focused=",
-        this.focused
-      );
+      this.debug("draw", coding, img_data, " at ", `${x},${y}`, ") focused=", this.focused);
     } else {
-      this.debug(
-        "draw",
-        "do_paint(",
-        img_data.length,
-        " bytes of",
-        coding,
-        " data ",
-        width,
-        "x",
-        height,
-        " at ",
-        x,
-        ",",
-        y,
-        ") focused=",
-        this.focused
-      );
+      this.debug("draw", "do_paint(", img_data.length, " bytes of", coding, " data ", width, "x", height,
+        " at ", x, ",", y, ") focused=", this.focused);
     }
 
     function painted(skip_box) {
       me.paint_pending = 0;
       if (!skip_box && me.debug_categories.includes("draw")) {
         const color = DEFAULT_BOX_COLORS[coding] || "white";
-        this.paint_box(color, x, y, width, height);
+        me.paint_box(color, x, y, width, height);
       }
       decode_callback();
     }
@@ -1463,19 +1334,12 @@ class XpraWindow {
           return;
         }
         const rgb_data = decode_rgb(packet);
-        const img = this.offscreen_canvas_ctx.createImageData(
-          enc_width,
-          enc_height
-        );
+        const img = this.offscreen_canvas_ctx.createImageData(enc_width, enc_height);
         img.data.set(rgb_data);
         this.offscreen_canvas_ctx.putImageData(img, x, y, 0, 0, width, height);
         painted();
         this.may_paint_now();
-      } else if (
-        coding == "jpeg" ||
-        coding.startsWith("png") ||
-        coding == "webp"
-      ) {
+      } else if (coding == "jpeg" || coding.startsWith("png") || coding == "webp") {
         if (bitmap) {
           paint_bitmap();
           return;
@@ -1512,14 +1376,8 @@ class XpraWindow {
           const ydelta = scroll_data[5];
           this.offscreen_canvas_ctx.drawImage(
             this.draw_canvas,
-            sx,
-            sy,
-            sw,
-            sh,
-            sx + xdelta,
-            sy + ydelta,
-            sw,
-            sh
+            sx, sy, sw, sh,
+            sx + xdelta, sy + ydelta, sw, sh
           );
           if (this.debug_categories.includes("draw")) {
             this.paint_box("brown", sx + xdelta, sy + ydelta, sw, sh);
